#!/usr/bin/python3

import numpy as np
import matplotlib as mpl
mpl.use('Agg')                    # For plotting without X server
import matplotlib.pyplot as plt
import sys
import os.path

# [TO-DO FOR USER, will not work without correct path]
# Add path to densmap code (be vary to use the correct version)
sys.path.append('/.../.../densmap')

import densmap as dm

# Set the folder name
MDname = "Flow"
# Time in ps of each bin
binT = 12.5

# Get the number of files
Nmd = len([f for f in os.listdir(MDname) if os.path.isfile(os.path.join(MDname, f))])

# Predefined average bulk density, measured by Petter at some point [kg/m3], also reported in our first paper
blkDen = 986
# fraction of bulk/mean density to define the interface point
dItf = 0.5
# Number of bins to skip (use rough estimate, just skip few points from the wall)
nSkp = 7

# For testing interpolation of instantaneous interface shape
OrdS  = 5
nSkpI = 4+3        # Use same number as for the averaged interface
yWb   =  0.706928
yWt   = 29.927073

# Read the first file, get display some info
# Read the first file, display some outputs
MDflow, info = dm.read_data(MDname+"/flow_%05d.dat"%1)
Nx  = info['shape'][0]
Ny  = info['shape'][1]
print("Read file with Nx = %d, Ny = %d bins! Measurement units: nm"%(Nx,Ny))
Xar = np.array( MDflow['X'] )
Yar = np.array( MDflow['Y'] )
Xar = Xar.reshape((Nx,Ny))
Yar = Yar.reshape((Nx,Ny))
bin_dx = Xar[1,0]-Xar[0,0]
bin_dy = Yar[0,1]-Yar[0,0]
# Display the minimum and maximum binning coordinates
print("x in [%f, %f], dx = %f"%(Xar[0,0],Xar[Nx-1,0],bin_dx))
print("y in [%f, %f], dy = %f"%(Yar[0,0],Yar[0,Ny-1],bin_dy))

# For GROMACS version 2020
bin_dz = 4.67650
densScale = 1.6605402/(bin_dx*bin_dy*bin_dz)

# # For GROMACS version 2021
# densScale = 1.6605402

# Arrays to hold the interface location data
IntfData = np.zeros((Ny-2*nSkpI,4))
IntfDataR= np.zeros((Ny-2*nSkpI,2))

# Extract the CL coordinate data
DropCL = np.zeros((Nmd,5))
DropIdx= np.zeros((Nmd,2))
for k in range(1,Nmd+1):
    # Read the MD data
    MDflow, info = dm.read_data(MDname+"/flow_%05d.dat"%k)
    # Reshape needed data into rectangular arrays
    Xar = Xar.reshape((Nx,Ny))
    # Reshape and re-normalise the mass array into density array [kg/m3]
    Mar = np.array( MDflow['M'] )
    Mar = Mar.reshape((Nx,Ny))*densScale
    # Current time
    DropCL [k-1,0] = k*binT
    DropIdx[k-1,0] = k*binT
    # Left displacement
    vInd = nSkp
    iR   = (Mar[:,vInd]>dItf*blkDen).nonzero()[0][ 0]; iL = iR-1;
    try:
        xBotL =  Xar[iL,vInd] + (Xar[iR,vInd]-Xar[iL,vInd])*(dItf*blkDen-Mar[iL,vInd])/(Mar[iR,vInd]-Mar[iL,vInd])
    except IndexError:
        print("Weird, something did not work, left-bottom!")
    vInd = Ny-1-nSkp
    iR   = (Mar[:,vInd]>dItf*blkDen).nonzero()[0][ 0]; iL = iR-1;
    try:
        xTopL =  Xar[iL,vInd] + (Xar[iR,vInd]-Xar[iL,vInd])*(dItf*blkDen-Mar[iL,vInd])/(Mar[iR,vInd]-Mar[iL,vInd])
    except IndexError:
        print("Weird, something did not work, left-top!")
    DropCL[k-1,1] = xTopL
    DropCL[k-1,2] = xBotL
    # Right displacement
    vInd = nSkp
    iL   = (Mar[:,vInd]>dItf*blkDen).nonzero()[0][-1]; iR = iL+1;
    try:
        xBotR =  Xar[iL,vInd] + (Xar[iR,vInd]-Xar[iL,vInd])*(dItf*blkDen-Mar[iL,vInd])/(Mar[iR,vInd]-Mar[iL,vInd])
    except IndexError:
        print("Weird, something did not work, right-bottom!")
    vInd = Ny-1-nSkp
    iL   = (Mar[:,vInd]>dItf*blkDen).nonzero()[0][-1]; iR = iL+1;
    try:
        xTopR =  Xar[iL,vInd] + (Xar[iR,vInd]-Xar[iL,vInd])*(dItf*blkDen-Mar[iL,vInd])/(Mar[iR,vInd]-Mar[iL,vInd])
    except IndexError:
        print("Weird, something did not work, right-top!")
    DropCL[k-1,3] = xTopR
    DropCL[k-1,4] = xBotR
    # ------
    # Now comes the interpolation part
    # ------
    # Carry out interface extraction and interpolation
    dAvg = dItf
    for i in range(Ny-2*nSkpI):
        # Compute slice average
        iL   = (Mar[:,nSkpI+i]>dAvg*blkDen).nonzero()[0][ 0]+10;
        iR   = (Mar[:,nSkpI+i]>dAvg*blkDen).nonzero()[0][-1]-10;
        avgS = np.sum(0.5*(Mar[iL+1:iR+1,nSkpI+i]+Mar[iL:iR,nSkpI+i])*np.diff(Xar[iL:iR+1,nSkpI+i]))/(Xar[iR,nSkpI+i]-Xar[iL,nSkpI+i])
        # Now work on the data
        IntfData[i,1] = Yar[0,nSkpI+i]
        iR = (Mar[:,nSkpI+i]>dItf*avgS).nonzero()[0][ 0]; iL = iR-1;
        IntfData[i,0] = Xar[iL,nSkpI+i] + (Xar[iR,nSkpI+i]-Xar[iL,nSkpI+i])*(dItf*avgS-Mar[iL,nSkpI+i])/(Mar[iR,nSkpI+i]-Mar[iL,nSkpI+i])
        IntfDataR[i,1] = Yar[0,nSkpI+i]
        iL = (Mar[:,nSkpI+i]>dItf*avgS).nonzero()[0][-1]; iR = iL+1;
        IntfDataR[i,0] = Xar[iL,nSkpI+i] + (Xar[iR,nSkpI+i]-Xar[iL,nSkpI+i])*(dItf*avgS-Mar[iL,nSkpI+i])/(Mar[iR,nSkpI+i]-Mar[iL,nSkpI+i])
    # Mirror the right profile
    for i in range(len(IntfDataR[:,0])):
        IntfDataR[i,0] =-IntfDataR[i,0]
        IntfDataR[i,1] =-IntfDataR[i,1]
    # Sort by the y coordinates
    IntfDataR[:,1] = IntfDataR[:,1] + 35.0
    IntfDataR.view('i8,i8').sort(order=['f1'], axis=0)
    # Put the mirrored right profile in the data array
    IntfData[:,2] = IntfDataR[:,0]
    # Compute the average
    IntfData[:,3] = 0.5*(IntfData[:,0]+IntfData[:,2])
    p0 = np.polyfit(IntfData[:,1],IntfData[:,3],OrdS)
    fit_eq = np.poly1d(p0)
    DropIdx[k-1,1] = fit_eq(yWt)-fit_eq(yWb)
    print("Done file %d!"%k)

# Save the data file
np.savetxt("00_get_CL_coordinates.txt", DropCL, header="t [ps], xcl top-left [nm], xcl bot-left [nm], xcl top-right [nm], xcl bot-right [nm]")
np.savetxt("00_get_CL_coordinates_Idx.txt", DropIdx, header="t [ps], dx [nm]")

# Plot the coordinates and/or displacement in the next script, for quick re-plot to get average

