#!/usr/bin/python3

import numpy as np
import matplotlib as mpl
mpl.use('Agg')                    # For plotting without X server
import matplotlib.pyplot as plt
import sys
import os.path

# [TO-DO FOR USER, will not work without correct path]
# Add path to densmap code (be vary to use the correct version)
sys.path.append('/.../.../densmap')

import densmap as dm

# Filter to process the closest to wall results
import scipy.signal as sg

plt.rcParams['text.usetex']=True
plt.rcParams.update({'figure.autolayout': True})  # to make sure that labels are inside printing area

font = {'family' : 'DejaVu Sans',
        'weight' : 'bold',
        'size'   : 18}

plt.rc('font', **font)

# Number of bins to skip, purely based on how much density we have accessible near the wall
nSkp = 3

# Set the folder name
MDname = "Flow"
# Set the start index for the averaging
iStart = 481
# iStart = 1

# Get the total number of files
Nmd = len([f for f in os.listdir(MDname) if os.path.isfile(os.path.join(MDname, f))])

# Nmd = Nmd-2 # If there are extra files
# # Override if needed
# Nmd = 634
# Nmd = iStart+65

# fraction of bulk/mean density to define the interface point
dItf = 0.5
# Bulk density in kg/m3
blkDen = 986

# Read slice density distribution over height
DensY = np.loadtxt("plotLiqDens_vs_y.txt")

# Read the first file, display some outputs
MDflow, info = dm.read_data(MDname+"/flow_%05d.dat"%iStart)
Nx  = info['shape'][0]
Ny  = info['shape'][1]
print("Read file with Nx = %d, Ny = %d bins! Measurement units: nm"%(Nx,Ny))
Xar = np.array( MDflow['X'] )
Yar = np.array( MDflow['Y'] )
Xar = Xar.reshape((Nx,Ny))
Yar = Yar.reshape((Nx,Ny))
# Display the minimum and maximum binning coordinates
bin_dx = Xar[1,0]-Xar[0,0]
bin_dy = Yar[0,1]-Yar[0,0]
print("x in [%f, %f], dx = %f"%(Xar[0,0],Xar[Nx-1,0],bin_dx))
print("y in [%f, %f], dy = %f"%(Yar[0,0],Yar[0,Ny-1],bin_dy))


# For GROMACS version 2020
bin_dz = 4.67650
densScale = 1.6605402/(bin_dx*bin_dy*bin_dz)

# # For GROMACS version 2021
# densScale = 1.6605402

# Initiate the sum matrix (all following snap-shots will be interpolated to match the first mass centrum)
MarS = np.array( MDflow['M'] )
MarS = MarS.reshape((Nx,Ny))*densScale
# Initiate the velocity matrices
UarS = np.array( MDflow['U'] ); UarS = UarS.reshape((Nx,Ny))/(Nmd+1-iStart)
VarS = np.array( MDflow['V'] ); VarS = VarS.reshape((Nx,Ny))/(Nmd+1-iStart)

# Construct array of left interface x coordinates, our reference frame to work with
DataItf = np.zeros((Ny,2))
DataXLo = np.zeros(Ny)
DataItf[:,0] = Yar[0,:]
for i in range(Ny-2*nSkp):
    idY = nSkp+i
    avgS = DensY[idY,1]
    if(i==0 or i== Ny-2*nSkp-1):
        MarS[0,idY] = 0; MarS[Nx-1,idY] = 0; # Making sure that there are no spikes at the end of domain
        Mslice = sg.savgol_filter(MarS[:,idY], 15, 4) # Filter the signal, window 15, order 4
        iL = (Mslice>dItf*avgS).nonzero()[0][-1]; iR = iL+1;
        DataItf[idY,1] = Xar[iL,idY] + (Xar[iR,idY]-Xar[iL,idY])*(dItf*avgS-Mslice[iL])/(Mslice[iR]-Mslice[iL])
    else:
        iL = (MarS[:,idY]>dItf*avgS).nonzero()[0][-1]; iR = iL+1;
        DataItf[idY,1] = Xar[iL,idY] + (Xar[iR,idY]-Xar[iL,idY])*(dItf*avgS-MarS[iL,idY])/(MarS[iR,idY]-MarS[iL,idY])
    print("At idY = %d obtained interface coordinate %.2f nm"%(idY,DataItf[idY,1]))
    

# Loop over all files in steady regime and and compute the density matrix sum
for k in range(iStart+1,Nmd+1):
    # Read the MD data
    MDflow, info = dm.read_data(MDname+"/flow_%05d.dat"%k)
    # Reshape and re-normalise the mass array into density array [kg/m3]
    Mar = np.array( MDflow['M'] )
    Mar = Mar.reshape((Nx,Ny))*densScale
    Uar = np.array( MDflow['U'] ); Uar = Uar.reshape((Nx,Ny));
    Var = np.array( MDflow['V'] ); Var = Var.reshape((Nx,Ny));    
    # Compute the current left contact line coordinates and shift the slices to target CL coordinate
    for i in range(Ny-2*nSkp):
        idY = nSkp+i
        Mslice = Mar[:,idY]; # Array for getting the interface location, either filtered or not
        avgS = DensY[idY,1]
        if(i==0 or i== Ny-2*nSkp-1):
            Mar[0,idY] = 0; Mar[Nx-1,idY] = 0; # Making sure that there are no spikes at the end of domain
            Mslice = sg.savgol_filter(Mar[:,idY], 15, 4) # Filter the signal, window 15, order 4
        iL = (Mslice>dItf*avgS).nonzero()[0][-1]; iR = iL+1;
        xLc= Xar[iL,idY] + (Xar[iR,idY]-Xar[iL,idY])*(dItf*avgS-Mslice[iL])/(Mslice[iR]-Mslice[iL])
        # Compute magnitude of the interface velocity in nm/ns or m/s
        Uitf = (xLc-DataXLo[idY])/0.0125
        # Store in old array
        DataXLo[idY] = xLc
        # Shift the slices
        intS = int((DataItf[idY,1]-xLc)/bin_dx) - 1
        remS =     (DataItf[idY,1]-xLc)         - intS*bin_dx
        MslL = np.roll(Mar[:,idY],intS  )
        MslR = np.roll(Mar[:,idY],intS+1)
        Msl  = MslL + (MslR - MslL)*remS/bin_dx
        MarS[:,idY] = MarS[:,idY] + Msl
        UslL = np.roll(Uar[:,idY],intS); UslR = np.roll(Uar[:,idY],intS+1); Usl  = UslL + (UslR - UslL)*remS/bin_dx
        VslL = np.roll(Var[:,idY],intS); VslR = np.roll(Var[:,idY],intS+1); Vsl  = VslL + (VslR - VslL)*remS/bin_dx
        UarS[:,idY]= UarS[:,idY] + Usl/(Nmd+1-iStart)
        VarS[:,idY]= VarS[:,idY] + Vsl/(Nmd+1-iStart)
        # Recompute the left contact line location
        Mslice = Msl
        if(i==0 or i== Ny-2*nSkp-1):
            Msl[0] = 0; Msl[Nx-1] = 0; # Making sure that there are no spikes at the end of domain
            Mslice = sg.savgol_filter(Msl, 15, 4) # Filter the signal, window 15, order 4
        iL = (Mslice>dItf*avgS).nonzero()[0][-1]; iR = iL+1;
        xLr = Xar[iL,idY] + (Xar[iR,idY]-Xar[iL,idY])*(dItf*avgS-Mslice[iL])/(Mslice[iR]-Mslice[iL])
        print("(file %05d, idY = %03d) avgS = %f, Target xL = %f, recomputed xL = %f, Uitf = %f m/s!"%(k,idY,avgS,DataItf[idY,1],xLr,Uitf))
        # print("(file %05d, idY = %03d) avgS = %f, Target xL = %f, original xL = %f, recomputed xL = %f!"%(k,idY,avgS,DataItf[idY,1],xLc,xLr))
        

# Get the average matrices
Mar = MarS/(Nmd+1-iStart)
# Uar = UarS/(Nmd+1-iStart)
# Var = VarS/(Nmd+1-iStart)
Uar = UarS
Var = VarS

# Save the averaged data
np.savetxt("04_average_wItfRight_XitfCoord.txt",DataItf,header="Reference X coordinates\ny [nm], x [nm]")
np.savetxt("04_average_wItfRight_Mmatrix.txt", Mar,header="Averaged M data over %d files, %f ns"%(Nmd+1-iStart,(Nmd+1-iStart)*12.5/1000))
np.savetxt("04_average_wItfRight_Umatrix.txt", Uar,header="Averaged U data over %d files, %f ns"%(Nmd+1-iStart,(Nmd+1-iStart)*12.5/1000))
np.savetxt("04_average_wItfRight_Vmatrix.txt", Var,header="Averaged V data over %d files, %f ns"%(Nmd+1-iStart,(Nmd+1-iStart)*12.5/1000))
