#!/usr/bin/python3

import numpy as np
import matplotlib.pyplot as plt
import sys

plt.rcParams['text.usetex']=True
plt.rcParams.update({'figure.autolayout': True})  # to make sure that labels are inside printing area

font = {'family' : 'DejaVu Sans',
        'weight' : 'bold',
        'size'   : 18}

plt.rc('font', **font)

# Define the simulation parameters
CaNum = "Ca = 0.20"
ChNum = "e = 0.60"
Uw    = 20/3

# Set the vertical bin index to use
idY = 3

# Turn on or off interface location extraction
pltItf = 1

# fraction of bulk/mean density to define the interface point
dItf = 0.5
dAvg = 0.10 # This is over-riden later in the file
cAvg = 0.10 # Criterion for slice density average computation close to wall
# Bulk density in kg/m3
blkDen = 986

# Read the bin array data
Xar = np.loadtxt("../02_Xmatrix.txt")
Yar = np.loadtxt("../02_Ymatrix.txt")
Nx  = np.shape(Xar)[0]
Ny  = np.shape(Xar)[1]
bin_dx = Xar[1,0]-Xar[0,0]

# Define symmetric index for top wall
idYt = Ny-1-idY

print("Read file with Nx = %d, Ny = %d bins! Measurement units: nm"%(Nx,Ny))

# Read the density and velocity array
Mar = np.loadtxt("../02_average_wMcent_Mmatrix.txt")
Uar = np.loadtxt("../02_average_wMcent_Umatrix.txt")
Var = np.loadtxt("../02_average_wMcent_Vmatrix.txt")

# Read the left aligned velocity results
CrdL = np.loadtxt("../03_average_wItfLeft_XitfCoord.txt")
UarL = np.loadtxt("../03_average_wItfLeft_Umatrix.txt")
VarL = np.loadtxt("../03_average_wItfLeft_Vmatrix.txt")

# Read the right aligned velocity results
CrdR = np.loadtxt("../04_average_wItfRight_XitfCoord.txt")
UarR = np.loadtxt("../04_average_wItfRight_Umatrix.txt")
VarR = np.loadtxt("../04_average_wItfRight_Vmatrix.txt")

# Transition from nm/ps to m/s
Uar = Uar*1000; UarL = UarL*1000; UarR = UarR*1000
Var = Var*1000; VarL = VarL*1000; VarR = VarR*1000

# Display the minimum and maximum binning coordinates
print("x in [%f, %f], dx = %f"%(Xar[0,0],Xar[Nx-1,0],Xar[1,0]-Xar[0,0]))
print("y in [%f, %f], dy = %f"%(Yar[0,0],Yar[0,Ny-1],Yar[0,1]-Yar[0,0]))

if pltItf == 1:
    # Compute slice average
    iL   = (Mar[:,idY]>dAvg*blkDen).nonzero()[0][ 0]+10;
    iR   = (Mar[:,idY]>dAvg*blkDen).nonzero()[0][-1]-10;
    avgS = np.sum(0.5*(Mar[iL+1:iR+1,idY]+Mar[iL:iR,idY])*np.diff(Xar[iL:iR+1,idY]))/(Xar[iR,idY]-Xar[iL,idY])
    # Get the interface coordinates
    iR = (Mar[:,idY]>dItf*avgS).nonzero()[0][ 0]; iL = iR-1;
    xL = Xar[iL,idY] + (Xar[iR,idY]-Xar[iL,idY])*(dItf*avgS-Mar[iL,idY])/(Mar[iR,idY]-Mar[iL,idY])
    iL = (Mar[:,idY]>dItf*avgS).nonzero()[0][-1]; iR = iL+1;
    xR = Xar[iL,idY] + (Xar[iR,idY]-Xar[iL,idY])*(dItf*avgS-Mar[iL,idY])/(Mar[iR,idY]-Mar[iL,idY])
    # Do the same for top wall
    iL   = (Mar[:,idYt]>dAvg*blkDen).nonzero()[0][ 0]+10;
    iR   = (Mar[:,idYt]>dAvg*blkDen).nonzero()[0][-1]-10;
    avgS = np.sum(0.5*(Mar[iL+1:iR+1,idYt]+Mar[iL:iR,idYt])*np.diff(Xar[iL:iR+1,idYt]))/(Xar[iR,idYt]-Xar[iL,idYt])
    # Get the interface coordinates
    iR = (Mar[:,idYt]>dItf*avgS).nonzero()[0][ 0]; iL = iR-1;
    xLt= Xar[iL,idYt] + (Xar[iR,idYt]-Xar[iL,idYt])*(dItf*avgS-Mar[iL,idYt])/(Mar[iR,idYt]-Mar[iL,idYt])
    iL = (Mar[:,idYt]>dItf*avgS).nonzero()[0][-1]; iR = iL+1;
    xRt= Xar[iL,idYt] + (Xar[iR,idYt]-Xar[iL,idYt])*(dItf*avgS-Mar[iL,idYt])/(Mar[iR,idYt]-Mar[iL,idYt])

# Get the shifts for the left centered velocity profiles
xLs  = xL -CrdL[idY ,1]
xLst = xLt-CrdL[idYt,1]
# Find indices for truncation
nPlot = 50 # Plot nPlot bins
iCL = (Xar[:,idY]>CrdL[idY ,1]).nonzero()[0][ 0];
iCLt= (Xar[:,idY]>CrdL[idYt,1]).nonzero()[0][ 0];

# Get the shifts for the right centered velocity profiles
xRs  = xR -CrdR[idY ,1]
xRst = xRt-CrdR[idYt,1]
# Find indices for truncation
iCR = (Xar[:,idY]>CrdR[idY ,1]).nonzero()[0][ 0];
iCRt= (Xar[:,idY]>CrdR[idYt,1]).nonzero()[0][ 0];

    
# Do the plotting, save figure
plt.figure(1, figsize=(12, 6.75), dpi=80)
plt.plot(Xar[:,idY],Uar[:,idY],'g-',label="MD measurement")
plt.plot(Xar[iCL:iCL+nPlot,idY]+xLs,UarL[iCL:iCL+nPlot,idY],'y-')
plt.plot(Xar[iCR-nPlot:iCR,idY]+xRs,UarR[iCR-nPlot:iCR,idY],'m-')
# plt.plot(Xar[:,idY],Mar[:,idY]/np.max(Mar[:,idY])*np.max(Uar[:,idY]),'-',color='silver')
# plt.plot(Xar[:,idY],Mar[:,idY],'-',color='silver')
plt.plot([np.min(Xar[:,idY]),np.max(Xar[:,idY])],[-Uw,-Uw],'r--',label="Bottom wall velocity")
if pltItf == 1:
    plt.plot([xL,xL],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:',label="Interface")
    plt.plot([xR,xR],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:')
plt.title(r"Velocity slice (avg. w.r.t $x_c$), bin coord. y = %.4f nm, %s, %s"%(Yar[0,idY],CaNum,ChNum))
plt.xlabel("x [nm]")
plt.ylabel(r"$u_x$ [m/s]")
plt.legend(loc="upper center")
# plt.ylim([-7.0,7.5])
plt.xlim([45.0,100])
plt.savefig("Slice_Uvel_yind%03d.pdf"%idY, format="pdf")

plt.figure(2, figsize=(12, 6.75), dpi=80)
plt.plot(Xar[:,idYt],Uar[:,idYt],'g-',label="MD measurement")
plt.plot(Xar[iCLt:iCLt+nPlot,idYt]+xLst,UarL[iCLt:iCLt+nPlot,idYt],'y-')
plt.plot(Xar[iCRt-nPlot:iCRt,idYt]+xRst,UarR[iCRt-nPlot:iCRt,idYt],'m-')
# plt.plot(Xar[:,idYt],Mar[:,idYt]/np.max(Mar[:,idYt])*np.max(Uar[:,idYt]),'-',color='silver')
# plt.plot(Xar[:,idYt],Mar[:,idYt],'-',color='silver')
plt.plot([np.min(Xar[:,idYt]),np.max(Xar[:,idYt])],[Uw,Uw],'r--',label="Top wall velocity")
if pltItf == 1:
    plt.plot([xLt,xLt],[np.min(Uar[:,idYt]),np.max(Uar[:,idYt])],'k:',label="Interface")
    plt.plot([xRt,xRt],[np.min(Uar[:,idYt]),np.max(Uar[:,idYt])],'k:')
plt.title(r"Velocity slice (avg. w.r.t $x_c$), bin coord. y = %.4f nm, %s, %s"%(Yar[0,idYt],CaNum,ChNum))
plt.xlabel("x [nm]")
plt.ylabel(r"$u_x$ [m/s]")
plt.legend(loc="lower center")
# plt.ylim([-7.0,7.5])
plt.xlim([55.0,110])
plt.savefig("Slice_Uvel_yind%03d.pdf"%(idYt), format="pdf")


# Domain center x coordinate
Xc = 0.5*(Xar[0,0]+Xar[-1,0])

# Flip the top-wall velocity data (these are coordinate aligned)
UarTf = - np.flip(Uar[:,idYt])
VarTf = - np.flip(Var[:,idYt])
# Create averaged arrays (easy)
UarTBa = np.copy(Uar[:,idY])
UarTBa = 0.5*(UarTBa+UarTf)
VarTBa = np.copy(Var[:,idY])
VarTBa = 0.5*(VarTBa+VarTf)

# Flip the interface aligned data
# First, coordinates
xLf  = 2*Xc - CrdR[idYt,1]
xRf  = 2*Xc - CrdL[idYt,1] # Flipped coordinates of the aligned results
# Second, needed shift to match the global interface location
xLsf = xL - xLf
xRsf = xR - xRf
# Third, flip actual velocity results (note, also left -> right and right -> left)
UarLf = - np.flip(UarR[:,idYt])
VarLf = - np.flip(VarR[:,idYt])
UarRf = - np.flip(UarL[:,idYt])
VarRf = - np.flip(VarL[:,idYt])
# Fourth, find indices for truncation (new ones)
iCLf = (Xar[:,idY]> xLf).nonzero()[0][ 0];
iCRf = (Xar[:,idY]> xRf).nonzero()[0][ 0];
# Fifth, create averaged arrays (harder, need to interpolate to coordinates of bottom velocity)
UarLa = np.copy(UarL[:,idY])
intS  = int((xLsf-xLs)/bin_dx) - 1             # This needs to be taken into account also in MD post-proc!!!!!
remS  =     (xLsf-xLs)         - intS*bin_dx
UitpL = np.roll(UarLf,intS  )
UitpR = np.roll(UarLf,intS+1)
Uitp  = UitpL + (UitpR - UitpL)*remS/bin_dx
UarLa = 0.5*(UarLa + Uitp)
# Vcomp
VarLa = np.copy(VarL[:,idY])
VitpL = np.roll(VarLf,intS  )
VitpR = np.roll(VarLf,intS+1)
Vitp  = VitpL + (VitpR - VitpL)*remS/bin_dx
VarLa = 0.5*(VarLa + Vitp)
# Right interface
UarRa = np.copy(UarR[:,idY])
intS  = int((xRsf-xRs)/bin_dx) - 1             # This needs to be taken into account also in MD post-proc!!!!!
remS  =     (xRsf-xRs)         - intS*bin_dx
UitpL = np.roll(UarRf,intS  )
UitpR = np.roll(UarRf,intS+1)
Uitp  = UitpL + (UitpR - UitpL)*remS/bin_dx
UarRa = 0.5*(UarRa + Uitp)
# Vcomp
VarRa = np.copy(VarR[:,idY])
VitpL = np.roll(VarRf,intS  )
VitpR = np.roll(VarRf,intS+1)
Vitp  = VitpL + (VitpR - VitpL)*remS/bin_dx
VarRa = 0.5*(VarRa + Vitp)


# Plot the velocity profiles from top and bottom along each other, central velocity
plt.figure(3, figsize=(12, 6.75), dpi=80)
plt.plot(Xar[:,idY],Uar[:,idY],'g-',label="MD measurement")
plt.plot(Xar[:,idY],UarTf,'r--')
plt.plot(Xar[:,idY],UarTBa,'m:')
if pltItf == 1:
    plt.plot([xL,xL],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:',label="Interface")
    plt.plot([2*Xc-xLt,2*Xc-xLt],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:')
    plt.plot([xR,xR],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:')
    plt.plot([2*Xc-xRt,2*Xc-xRt],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:')
plt.title(r"Velocity slice (avg. w.r.t $x_c$), bin coord. y = %.4f nm, %s, %s"%(Yar[0,idYt],CaNum,ChNum))
plt.xlabel("x [nm]")
plt.ylabel(r"$u_x$ [m/s]")
plt.legend(loc="upper center")
# plt.ylim([-7.0,7.5])
plt.xlim([45.0,100])

# Left/right aligned velocity
plt.figure(4, figsize=(12, 6.75), dpi=80)
plt.plot(Xar[iCL:iCL+nPlot,idY]+xLs,UarL[iCL:iCL+nPlot,idY],'r-')
plt.plot(Xar[iCLf:iCLf+nPlot,idY]+xLsf,UarLf[iCLf:iCLf+nPlot],'g--')
plt.plot(Xar[iCL:iCL+nPlot,idY]+xLs,UarLa[iCL:iCL+nPlot],'c:')
plt.plot(Xar[iCR-nPlot:iCR,idY]+xRs,UarR[iCR-nPlot:iCR,idY],'y-')
plt.plot(Xar[iCRf-nPlot:iCRf,idY]+xRsf,UarRf[iCRf-nPlot:iCRf],'m--')
plt.plot(Xar[iCR-nPlot:iCR,idY]+xRs,UarRa[iCR-nPlot:iCR],'c:')
if pltItf == 1:
    plt.plot([xL,xL],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:',label="Interface")
    plt.plot([xR,xR],[np.min(Uar[:,idY]),np.max(Uar[:,idY])],'k:')
plt.title(r"Velocity slice (avg. w.r.t $x_c$), bin coord. y = %.4f nm, %s, %s"%(Yar[0,idY],CaNum,ChNum))
plt.xlabel("x [nm]")
plt.ylabel(r"$u_x$ [m/s]")
plt.legend(loc="upper center")
# plt.ylim([-7.0,7.5])
plt.xlim([45.0,100])



# Trunkate vapour data and output the liquid part to file
if pltItf == 1:
    iL = (Xar[:,idY]>xL).nonzero()[0][ 0]+nPlot-1;
    iR = (Xar[:,idY]>xR).nonzero()[0][ 0]-nPlot+1;
    Ubot      = np.zeros((iR-iL,3))
    Ubot[:,1] = np.copy(UarTBa[iL:iR])
    Ubot[:,2] = np.copy(VarTBa[iL:iR])
    Ubot[:,0] = np.copy(Xar[iL:iR,idY])-Xc # Shift x to have 0 at the center of dom.
    np.savetxt("Slice_Uvel_tbAvg_yind%03d.txt"%(idY),Ubot,header="x [nm], U [m/s], V [m/s]")
    UbotL      = np.zeros((nPlot,3))
    UbotL[:,0] = np.copy(Xar[iCL:iCL+nPlot,idY])-Xc+xLs # Shifts
    UbotL[:,1] = np.copy(UarLa[iCL:iCL+nPlot])
    UbotL[:,2] = np.copy(VarLa[iCL:iCL+nPlot])
    np.savetxt("Slice_Uvel_tbAvg_yind%03d_left.txt"%(idY),UbotL,header="x [nm], U [m/s], V [m/s]")
    UbotR      = np.zeros((nPlot,3))
    UbotR[:,0] = np.copy(Xar[iCR-nPlot:iCR,idY])-Xc+xRs # Shifts
    UbotR[:,1] = np.copy(UarRa[iCR-nPlot:iCR])
    UbotR[:,2] = np.copy(VarRa[iCR-nPlot:iCR])
    np.savetxt("Slice_Uvel_tbAvg_yind%03d_right.txt"%(idY),UbotR,header="x [nm], U [m/s], V [m/s]")

    # Contact line positions
    DOut = np.array([xL-0.5*(Xar[0,0]+Xar[-1,0]),xR-0.5*(Xar[0,0]+Xar[-1,0])])
    np.savetxt("Slice_Uvel_yind%03d_MDclPos.txt"%(idY),DOut,header="xCLleft [nm], xCLright [nm]")
    


plt.show()
