#!/usr/bin/python3

import numpy as np
import matplotlib.pyplot as plt

plt.rcParams['text.usetex']=True
plt.rcParams.update({'figure.autolayout': True})  # to make sure that labels are inside printing area

font = {'family' : 'DejaVu Sans',
        'weight' : 'bold',
        'size'   : 24}

plt.rc('font', **font)

# Define the contact angle, in degrees, Ca number
caVal = 95.5
caStd = 4.3
CaNum = "Ca = 0.10"
ChNum = "e = 0.60 new param. 3"
# These coordinates are set based on first bins, where there is seemingly reliable velocity measurement
# yWb   =  0.906928
# yWt   = 29.727073
yWb   =  0.7054
yWt   = 29.9286

# Number of interface points to skip, based on equilibrium tests (run before and check)
nSkp = 4

# Read the MD data
MDdataAvg = {}
nMD=1
MDdataAvg[0] = np.loadtxt("steadyInterface_wMcent.txt")
MDlegend = ["Slice interface extraction"]
CurveSpec= ['g-.']

# Replace the x coordinate of MD results with the averaged one, cut the redundant rows
for i in range(nMD):
    MDdataAvg[i][:,0] = MDdataAvg[i][:, 3]
    MDdataAvg[i]      = MDdataAvg[i][:,:2]

# Cut away the points to neglect from the top and the bottom
if nSkp > 0:
    i = 0; MDdataAvg[i] = MDdataAvg[i][nSkp:-nSkp,:]

# Use reduced data set and try to predict the remaining data set. Loop over number of
# orders and exclusion points
Nord = 7 # Number of orders to consider
dspAr = np.zeros(Nord)
print(" --------------------- Cross-validation stats, interface shape ------------------")
print("  average prediction error")
print("  Ncr \ Ord", end='')
minE = 10.0
for OrdS in range(1,Nord+1):
    print(' %d'%OrdS, end='')
for Ncr in range(1,8):
    minEl= 10.0
    print("\n %d"%Ncr,end='')
    for OrdS in range(1,Nord+1):
        p0     = np.polyfit(MDdataAvg[0][Ncr:-Ncr,1],MDdataAvg[0][Ncr:-Ncr,0],OrdS)
        fit_eq = np.poly1d(p0)
        predB  = fit_eq(MDdataAvg[0][:Ncr ,1])
        predT  = fit_eq(MDdataAvg[0][-Ncr:,1])
        err    = np.sum(np.abs(MDdataAvg[0][:Ncr,0] - predB)) + np.sum(np.abs(MDdataAvg[0][-Ncr:,0] - predT))
        err    = 0.5*err/Ncr
        minE   = np.min([err,minE ])
        minEl  = np.min([err,minEl])
        print(" %.4f"%err,end='')
    print(" | %.4f"%minEl, end='')
print("\n  Minimum error = %.6f"%minE)
print(" --------------------------------------------------------------------------------")
    

# Compute interface angle (based on coordinate differences) and produce a plot of this
# First append all data arrays with extra columns for arch legth
# and angle information.
for i in range(nMD):
    MDdataAvg[i] = np.append(MDdataAvg[i][:,:],np.zeros(np.shape(MDdataAvg[i][:,0:2])),axis=1)
# Now, compute the curviliniar coordinate and angles
for i in range(nMD):
    for j in range(len(MDdataAvg[i][:,0])-1):
        dx = MDdataAvg[i][j+1,0]-MDdataAvg[i][j,0]
        dy = MDdataAvg[i][j+1,1]-MDdataAvg[i][j,1]
        MDdataAvg[i][j+1,2] = MDdataAvg[i][j,2] + np.sqrt(dx**2 + dy**2)
        MDdataAvg[i][j  ,3] = np.arctan2(dy,dx)
# Now, distribute the angles in a central way
for i in range(nMD):
    Ny = len(MDdataAvg[i][:,3])
    MDdataAvg[i][Ny-1,3] = MDdataAvg[i][Ny-2,3]
    MDdataAvg[i][1:-1,3] =(MDdataAvg[i][0:-2,3]+MDdataAvg[i][1:-1,3])*0.5


print(" --------------------- Cross-validation stats, interface angle ------------------")
print("  average prediction error")
print("  Ncr \ Ord", end='')
minE = 10.0
for OrdS in range(1,Nord+1):
    print(' %d'%OrdS, end='')
for Ncr in range(1,8):
    minEl= 10.0
    print("\n %d"%Ncr,end='')
    for OrdS in range(1,Nord+1):
        p0     = np.polyfit(MDdataAvg[0][Ncr:-Ncr,1],MDdataAvg[0][Ncr:-Ncr,3],OrdS)
        fit_eq = np.poly1d(p0)
        predB  = fit_eq(MDdataAvg[0][:Ncr ,1])
        predT  = fit_eq(MDdataAvg[0][-Ncr:,1])
        err    = np.sum(np.abs(MDdataAvg[0][:Ncr,3] - predB)) + np.sum(np.abs(MDdataAvg[0][-Ncr:,3] - predT))
        err    = 0.5*err/Ncr
        minE   = np.min([err,minE])
        minEl  = np.min([err,minEl])
        print(" %.4f"%err,end='')
    print(" | %.4f"%minEl, end='')
print("\n  Minimum error = %.6f"%minE)
print(" --------------------------------------------------------------------------------")


# The following checks are all done on the same Y coordinates as the MD data
Yfit = MDdataAvg[0][:,1]
# Same number that was used for cross-validation
Ncr  = 5

# Approximate the full interface shape with the selected optimal order
OrdS = 4
# Interpolation
p0     = np.polyfit(MDdataAvg[0][:,1],MDdataAvg[0][:,0],OrdS)
fit_eq = np.poly1d(p0)
Xfit   = fit_eq(Yfit)

# Approximate the full angle distribution with the selected optimal order
OrdA = 6
# Interpolation
p0     = np.polyfit(MDdataAvg[0][:,1],MDdataAvg[0][:,3],OrdA)
fit_eq = np.poly1d(p0)
Afit   = fit_eq(Yfit)

# Now check the cross-validation error, if we reconstruct the interface from the fitted angle data
# Constructing the interface
Ny = len(Xfit); dy = Yfit[1]-Yfit[0];
Ny2= int(Ny/2)
Xrec      = np.zeros(np.shape(Xfit))
Xrec[Ny2] = Xfit[Ny2]
for i in range(Ny2-1):
    Xrec[Ny2+i+1] = Xrec[Ny2+i] + dy/np.tan(Afit[Ny2+i+1])
    Xrec[Ny2-i-1] = Xrec[Ny2-i] - dy/np.tan(Afit[Ny2-i-1])
# A bit asymmetric, need to correct
Xrec[0] = Xrec[1] - dy/np.tan(Afit[0])
# Compute the cross validation error and output to screen
err = np.sum(np.abs(MDdataAvg[0][:Ncr,0] - Xrec[:Ncr])) + np.sum(np.abs(MDdataAvg[0][-Ncr:,0] - Xrec[-Ncr:]))
err2= np.sum(np.abs(MDdataAvg[0][:Ncr,0] - Xfit[:Ncr])) + np.sum(np.abs(MDdataAvg[0][-Ncr:,0] - Xfit[-Ncr:]))
print("  Cross validation error of reconstructed interface %.6f (direct error %.6f)"%(err,err2))

# Now check the cross-validation error, if we compute the interface angle from interface shape
XfitA  = np.copy(Xfit)
for j in range(len(Xfit)-1):
    dx = Xfit[j+1]-Xfit[j]
    dy = Yfit[j+1]-Yfit[j]
    XfitA[j] = np.arctan2(dy,dx)
Ny = len(Xfit)
XfitA[Ny-1] = XfitA[Ny-2]
XfitA[1:-1] =(XfitA[0:-2]+XfitA[1:-1])*0.5
# Now compute the error and display to screen
err = np.sum(np.abs(MDdataAvg[0][:Ncr,3] - XfitA[:Ncr])) + np.sum(np.abs(MDdataAvg[0][-Ncr:,3] - XfitA[-Ncr:]))
err2= np.sum(np.abs(MDdataAvg[0][:Ncr,3] - Afit [:Ncr])) + np.sum(np.abs(MDdataAvg[0][-Ncr:,3] - Afit [-Ncr:]))
print("  Cross validation error of computed angle distribution %.6f (direct error %.6f)"%(err,err2))


# Loop over orders to get the drop displacement predictions
Nord = 14
dspAr = np.zeros((2,Nord))
for OrdS in range(1,Nord+1):
    p0     = np.polyfit(MDdataAvg[0][:,1],MDdataAvg[0][:,0],OrdS)
    fit_eq = np.poly1d(p0)
    dspAr[0,OrdS-1] = OrdS
    dspAr[1,OrdS-1] = fit_eq(yWt)-fit_eq(yWb)

# Plot the displacement results with respect to order
plt.figure(1, figsize=(5.8/1.5, 6.75*0.95), dpi=80)
plt.plot(dspAr[0,:],dspAr[1,:],'k-x')
# plt.title("Extrapolated drop displacement")
plt.locator_params(axis='y', nbins=5)
plt.xlabel(r"$N_p$ [1]")
plt.ylabel(r"$\Delta x$, [nm]")
plt.savefig("forPaper_q2_Ca0.25_orderConv_v2.pdf", format="pdf")

plt.show()
