#!/usr/bin/python3

import numpy as np
import matplotlib.pyplot as plt

plt.rcParams['text.usetex']=True
plt.rcParams.update({'figure.autolayout': True})  # to make sure that labels are inside printing area

font = {'family' : 'DejaVu Sans',
        'weight' : 'bold',
        'size'   : 24}

plt.rc('font', **font)

# Define the contact angle, in degrees, Ca number
caVal = 95.0
caStd = 4.3
CaNum = "Ca = 0.10"
ChNum = "e = 0.60 new param. 3"
# yWb   =  0.906928
# yWt   = 29.727073
yWb   =  0.7054
yWt   = 29.9286
# Order of fit for interface and angle extrapolation
OrdS = 7
Ncr  = 5 # Number of points at each side for cross validations

# Number of interface points to skip, based on equilibrium tests
nSkp = 4

# Read the MD data
MDdataAvg = {}
nMD=1
MDdataAvg[0] = np.loadtxt("steadyInterface_wMcent.txt")
MDlegend = ["Slice interface extraction"]
CurveSpec= ['g-.']

# Replace the x coordinate of MD results with the averaged one, cut the redundant rows
for i in range(nMD):
    MDdataAvg[i][:,0] = MDdataAvg[i][:, 3]
    MDdataAvg[i]      = MDdataAvg[i][:,:2]

# Cut away the points to neglect from the top and the bottom
if nSkp > 0:
    i = 0; MDdataAvg[i] = MDdataAvg[i][nSkp:-nSkp,:]


# Carry out Ord-th-order polynomial fits of whole interface, get extrapolated drop displacement,
# plot zoomed views.

# First, fit the interface with reduced data set and try to predict remaining data set
p0     = np.polyfit(MDdataAvg[0][Ncr:-Ncr,1],MDdataAvg[0][Ncr:-Ncr,0],OrdS)
fit_eq = np.poly1d(p0)
predB  = fit_eq(MDdataAvg[0][:Ncr ,1])
predT  = fit_eq(MDdataAvg[0][-Ncr:,1])
err    = np.sum(np.abs(MDdataAvg[0][:Ncr,0] - predB)) + np.sum(np.abs(MDdataAvg[0][-Ncr:,0] - predT))
err    = 0.5*err/Ncr
print("Cross-validation, interface shape, error = %.6f"%err)
Yfit   = np.linspace(yWb,yWt,1000)
XfitC  = fit_eq(Yfit)

# Whole interface
p1, C_p = np.polyfit(MDdataAvg[0][:,1],MDdataAvg[0][:,0],OrdS,cov=True)
fit_eq  = np.poly1d(p1)
Xfit    = fit_eq(Yfit)

## Get the error estimate
# Source: https://stackoverflow.com/questions/28505008/numpy-polyfit-how-to-get-1-sigma-uncertainty-around-the-estimated-curve
# Matrix with rows 1, t, t**2, ...:
TT = np.vstack([Yfit**(OrdS-i) for i in range(OrdS+1)]).T
C_xi = np.dot(TT, np.dot(C_p, TT.T)) # C_y = TT*C_z*TT.T
sig_xi = np.sqrt(np.diag(C_xi))  # Standard deviations are sqrt of diagonal
XfitU = Xfit+3*sig_xi
XfitL = Xfit-3*sig_xi

# Now compute the interpolated drop displacement
drpDisp = Xfit[-1]-Xfit[0]

# Compute interface angle (based on coordinate differences) and produce a plot of this
# First append all data arrays with extra columns for arch legth
# and angle information.
for i in range(nMD):
    MDdataAvg[i] = np.append(MDdataAvg[i][:,:],np.zeros(np.shape(MDdataAvg[i][:,0:2])),axis=1)
# Now, compute the curviliniar coordinate and angles
for i in range(nMD):
    for j in range(len(MDdataAvg[i][:,0])-1):
        dx = MDdataAvg[i][j+1,0]-MDdataAvg[i][j,0]
        dy = MDdataAvg[i][j+1,1]-MDdataAvg[i][j,1]
        MDdataAvg[i][j+1,2] = MDdataAvg[i][j,2] + np.sqrt(dx**2 + dy**2)
        MDdataAvg[i][j  ,3] = np.arctan2(dy,dx)


# Create the angle array along with the appropriate coordinates (y,s,theta)
MDangleArr = {}
for i in range(nMD):
    MDangleArr[i] = np.zeros((len(MDdataAvg[i][:,0])-1,3))
    MDangleArr[i][:,0] = 0.5*(MDdataAvg[i][:-1,1]+MDdataAvg[i][1:,1])
    MDangleArr[i][:,1] = 0.5*(MDdataAvg[i][:-1,2]+MDdataAvg[i][1:,2])
    MDangleArr[i][:,2] =      MDdataAvg[i][:-1,3]
        
# # Now, distribute the angles in a central way - biased
# for i in range(nMD):
#     Ny = len(MDdataAvg[i][:,3])
#     MDdataAvg[i][Ny-1,3] = MDdataAvg[i][Ny-2,3]
#     MDdataAvg[i][1:-1,3] =(MDdataAvg[i][0:-2,3]+MDdataAvg[i][1:-1,3])*0.5

# Construct angle data from the fitted coordinate shape
Ny = len(Yfit)
Afit = np.zeros(Ny-1)
AfitU= np.zeros(Ny-1)
AfitL= np.zeros(Ny-1)
Sfit = np.zeros(Ny  )
SfitU= np.zeros(Ny  )
SfitL= np.zeros(Ny  )
for j in range(Ny-1):
    dx = Xfit[j+1]-Xfit[j]
    dxU= XfitU[j+1]-XfitU[j]
    dxL= XfitL[j+1]-XfitL[j]
    dy = Yfit[j+1]-Yfit[j]
    Sfit[j+1] = Sfit[j] + np.sqrt(dx**2 + dy**2)
    SfitU[j+1] = SfitU[j] + np.sqrt(dxU**2 + dy**2)
    SfitL[j+1] = SfitL[j] + np.sqrt(dxL**2 + dy**2)
    Afit[j  ] = np.arctan2(dy,dx)
    AfitU[j  ] = np.arctan2(dy,dxU)
    AfitL[j  ] = np.arctan2(dy,dxL)

# Construct the s and y arrays corresponding to angle computations
AfitY = 0.5*(Yfit[:-1]+Yfit[1:])
AfitS = 0.5*(Sfit[:-1]+Sfit[1:])
AfitSU= 0.5*(SfitU[:-1]+SfitU[1:])
AfitSL= 0.5*(SfitL[:-1]+SfitL[1:])
    
# Extract the extrapolated angle values
dynAdv = Afit[-1]/np.pi*180.0
dynRec = Afit[ 0]/np.pi*180.0

# Extract the angle values at the first and last reliable angle locations
yWbR = MDangleArr[0][ 0,0]
yWtR = MDangleArr[0][-1,0]
ind = (AfitY > yWbR).nonzero()[0][0]
dynRecR = Afit[ind]/np.pi*180.0
dynRecRu= AfitU[ind]/np.pi*180.0
dynRecRl= AfitL[ind]/np.pi*180.0
ind = (AfitY > yWtR).nonzero()[0][0]
dynAdvR = Afit[ind]/np.pi*180.0
dynAdvRu= AfitU[ind]/np.pi*180.0
dynAdvRl= AfitL[ind]/np.pi*180.0

# Print results to screen
print("The domain height for continuum simulations L = %.4f nm, yWb = %.4f nm"%(yWt-yWb,yWb))
print("Obtained extrapolated values (orderS %d)"%(OrdS))
print("   drop displacement: %.2f nm (dimless %.6f)"%(drpDisp,drpDisp/(yWt-yWb)))
print("    dyn. c.a. (adv.): %6.2f deg (eq. + %4.1f deg)"%(180-dynAdv,180-dynAdv-caVal))
print("    dyn. c.a. (rec.): %6.2f deg (eq. - %4.1f deg)"%(dynRec,caVal-dynRec))
print("  R dyn. c.a. (adv.): (%6.2f %6.2f %6.2f) deg (eq. + %4.1f deg)"%(180-dynAdvRl,180-dynAdvR,180-dynAdvRu,180-dynAdvR-caVal))
print("  R dyn. c.a. (rec.): (%6.2f %6.2f %6.2f) deg (eq. - %4.1f deg)"%(dynRecRl,dynRecR,dynRecRu,caVal-dynRecR))


#### Plotting section
# Do the plotting, save figure
plt.figure(1, figsize=(5.8, 6.75), dpi=80)
for i in range(nMD):
    plt.plot(MDdataAvg[i][:,0],MDdataAvg[i][:,1],CurveSpec[i],label=MDlegend[i])
plt.plot(Xfit,Yfit,'b:')
plt.title("Interface shape")
plt.xlabel("x [nm]")
plt.ylabel("y [nm]")
# plt.legend(loc="lower right")
# plt.legend(loc="lower right")
plt.axis("equal")
# plt.ylim([-25.0,25.0])
# plt.xlim([-2.0,5.0])
plt.savefig("steadyInterface_wMcent_eqAxis.pdf", format="pdf")



# Here comes zoomed plots
plt.figure(11, figsize=(5.8, 6.75), dpi=80)
for i in range(nMD):
    plt.plot(MDdataAvg[i][:,0],MDdataAvg[i][:,1],CurveSpec[i],label=MDlegend[i])
plt.plot([-5,5],[yWb,yWb],'r--')
plt.plot(Xfit,Yfit,'b:')
plt.title("Interface shape")
plt.xlabel("x [nm]")
plt.ylabel("y [nm]")
# plt.legend(loc="lower right")
# plt.legend(loc="lower right")
plt.axis("equal")
plt.ylim([0.5,3.18])
plt.xlim([-4.0,-1.69])
plt.savefig("steadyInterface_wMcent_eqAxis_zoom1.pdf", format="pdf")


plt.figure(12, figsize=(5.8, 6.75), dpi=80)
for i in range(nMD):
    plt.plot(MDdataAvg[i][:,0],MDdataAvg[i][:,1],CurveSpec[i],label=MDlegend[i])
plt.plot([0,10],[yWt,yWt],'r--')
plt.plot(Xfit,Yfit,'b:')
plt.title("Interface shape")
plt.xlabel("x [nm]")
plt.ylabel("y [nm]")
# plt.legend(loc="lower right")
# plt.legend(loc="lower right")
plt.axis("equal")
plt.ylim([27.0,30.1])
plt.xlim([2.3,5.0])
plt.savefig("steadyInterface_wMcent_eqAxis_zoom2.pdf", format="pdf")



# Plot the interface angle data
# plt.figure(2, figsize=(8.0, 4.75), dpi=80)
plt.figure(2, figsize=(12.5, 6.75), dpi=80)
for i in range(nMD):
    plt.plot(MDangleArr[i][:,0],MDangleArr[i][:,2]/np.pi*180.0,CurveSpec[i],label=MDlegend[i])
# i=0; plt.plot(MDdataAvg[i][3:-3,1],MDdataAvg[i][3:-3,3]/np.pi*180.0,CurveSpec[i],label=MDlegend[i])
# i=1; plt.plot(MDdataAvg[i][4:-4,1],MDdataAvg[i][4:-4,3]/np.pi*180.0,CurveSpec[i],label=MDlegend[i])
plt.plot(AfitY,Afit/np.pi*180.0,'b:')
plt.fill_between(AfitY, AfitU/np.pi*180.0, AfitL/np.pi*180.0, alpha=.25)
plt.scatter(np.array([yWb,yWt]),np.array([dynRec,dynAdv]),c='g',marker='x')
plt.scatter(np.array([yWbR,yWtR]),np.array([dynRecR,dynAdvR]),c='g',marker='x')
plt.plot([0 ,10],[    caVal      ,    caVal      ],'k:')
# plt.plot([0 ,10],[    caVal+caStd,    caVal+caStd],'k:')
# plt.plot([0 ,10],[    caVal-caStd,    caVal-caStd],'k:')
plt.plot([20,30],[180-caVal      ,180-caVal      ],'k:')
# plt.plot([20,30],[180-caVal+caStd,180-caVal+caStd],'k:')
# plt.plot([20,30],[180-caVal-caStd,180-caVal-caStd],'k:')
plt.title("Interface angle, MD, %s, %s"%(CaNum,ChNum))
plt.xlabel("y [nm]")
plt.ylabel(r"$\theta$ [deg]")
plt.legend(loc="upper right")
# plt.axis("equal")
# plt.ylim([65.0,115.0])
plt.savefig("steadyInterface_wMcent_angDist_y.pdf", format="pdf")


plt.show()
