#!/usr/bin/python3

import numpy as np
import matplotlib.pyplot as plt

plt.rcParams['text.usetex']=True
plt.rcParams.update({'figure.autolayout': True})  # to make sure that labels are inside printing area

font = {'family' : 'DejaVu Sans',
        'weight' : 'bold',
        'size'   : 24}

plt.rc('font', **font)

# Define the contact angle, in degrees, Ca number
caVal = 95.0
caStd = 4.0
CaNum = "Ca = 0.10"
ChNum = "e = 0.67"
# yWb   =  0.906928
# yWt   = 29.727073
yWb   =  0.7054
yWt   = 29.9286
# Order of fit for interface and angle extrapolation
OrdS = 7
Ncr  = 5 # Number of points at each side for cross validations

# Number of interface points to skip, based on equilibrium tests
nSkp = 4

# Read the MD data
MDdataAvg = {}
nMD=2
MDdataAvg[1] = np.loadtxt("steadyInterface_wMcent.txt")
MDdataAvg[0] = np.loadtxt("../../00_equil/02_average_wMcent/steadyInterface_wMcent.txt")
MDlegend = ["Equilibrium", "Sheared"]
CurveSpec= ['b-', 'g-']
# MDlegend = ["Sheared", "Equilibrium"]
# CurveSpec= ['g-', 'b-']

# Replace the x coordinate of MD results with the averaged one, cut the redundant rows
for i in range(nMD):
    MDdataAvg[i][:,0] = MDdataAvg[i][:, 3]
    MDdataAvg[i]      = MDdataAvg[i][:,:2]

# Cut away the points to neglect from the top and the bottom
if nSkp > 0:
    for i in range(nMD):
        MDdataAvg[i] = MDdataAvg[i][nSkp:-nSkp,:]


# Carry out Ord-th-order polynomial fits of whole interface, get extrapolated drop displacement,
# plot zoomed views.

# Fit the whole interface
i      = 1
Yfit   = np.linspace(yWb,yWt,1000)
p1     = np.polyfit(MDdataAvg[i][:,1],MDdataAvg[i][:,0],OrdS)
fit_eq = np.poly1d(p1)
Xfit   = fit_eq(Yfit)

# Additional fits to demonstrate the sensitivity
p1     = np.polyfit(MDdataAvg[i][:,1],MDdataAvg[i][:,0],OrdS+1)
fit_eq = np.poly1d(p1)
Xfit1  = fit_eq(Yfit)
p1     = np.polyfit(MDdataAvg[i][:,1],MDdataAvg[i][:,0],OrdS+4)
fit_eq = np.poly1d(p1)
Xfit2  = fit_eq(Yfit)
p1     = np.polyfit(MDdataAvg[i][:,1],MDdataAvg[i][:,0],OrdS-2)
fit_eq = np.poly1d(p1)
Xfit3  = fit_eq(Yfit)


# Now compute the interpolated drop displacement
drpDisp = Xfit[-1]-Xfit[0]

# Compute interface angle (based on coordinate differences) and produce a plot of this
# First append all data arrays with extra columns for arch legth
# and angle information.
for i in range(nMD):
    MDdataAvg[i] = np.append(MDdataAvg[i][:,:],np.zeros(np.shape(MDdataAvg[i][:,0:2])),axis=1)
# Now, compute the curviliniar coordinate and angles
for i in range(nMD):
    for j in range(len(MDdataAvg[i][:,0])-1):
        dx = MDdataAvg[i][j+1,0]-MDdataAvg[i][j,0]
        dy = MDdataAvg[i][j+1,1]-MDdataAvg[i][j,1]
        MDdataAvg[i][j+1,2] = MDdataAvg[i][j,2] + np.sqrt(dx**2 + dy**2)
        MDdataAvg[i][j  ,3] = np.arctan2(dy,dx)
        
MDangleArr = {}
for i in range(nMD):
    MDangleArr[i] = np.zeros((len(MDdataAvg[i][:,0])-1,3))
    MDangleArr[i][:,0] = 0.5*(MDdataAvg[i][:-1,1]+MDdataAvg[i][1:,1])
    MDangleArr[i][:,1] = 0.5*(MDdataAvg[i][:-1,2]+MDdataAvg[i][1:,2])
    MDangleArr[i][:,2] =      MDdataAvg[i][:-1,3]

# Construct angle data from the fitted coordinate shape
Ny = len(Yfit)
Afit = np.zeros(Ny-1)
Afit1= np.zeros(Ny-1)
Afit2= np.zeros(Ny-1)
Afit3= np.zeros(Ny-1)
Sfit = np.zeros(Ny  )
for j in range(Ny-1):
    dx = Xfit[j+1]-Xfit[j]
    dx1= Xfit1[j+1]-Xfit1[j]
    dx2= Xfit2[j+1]-Xfit2[j]
    dx3= Xfit3[j+1]-Xfit3[j]
    dy = Yfit[j+1]-Yfit[j]
    Sfit[j+1] = Sfit[j] + np.sqrt(dx**2 + dy**2)
    Afit[j  ] = np.arctan2(dy,dx)
    Afit1[j ] = np.arctan2(dy,dx1)
    Afit2[j ] = np.arctan2(dy,dx2)
    Afit3[j ] = np.arctan2(dy,dx3)

# Construct the s and y arrays corresponding to angle computations
AfitY = 0.5*(Yfit[:-1]+Yfit[1:])
AfitS = 0.5*(Sfit[:-1]+Sfit[1:])

# Extract the angle values
dynAdv = Afit[-1]/np.pi*180.0
dynRec = Afit[ 0]/np.pi*180.0

dynAdv1 = Afit1[-1]/np.pi*180.0
dynRec1 = Afit1[ 0]/np.pi*180.0
dynAdv2 = Afit2[-1]/np.pi*180.0
dynRec2 = Afit2[ 0]/np.pi*180.0

# Extract the angle values at the first and last reliable angle locations
yWbR = MDangleArr[0][ 0,0]
yWtR = MDangleArr[0][-1,0]
ind = (AfitY > yWbR).nonzero()[0][0]
dynRecR = Afit[ind]/np.pi*180.0
ind = (AfitY > yWtR).nonzero()[0][0]
dynAdvR = Afit[ind]/np.pi*180.0

# Print results to screen
print("The domain height for continuum simulations L = %.4f nm, yWb = %.4f nm"%(yWt-yWb,yWb))
print("Obtained extrapolated values (orderS %d)"%(OrdS))
print("   drop displacement: %.2f nm (dimless %.6f)"%(drpDisp,drpDisp/(yWt-yWb)))
print("    dyn. c.a. (adv.): %6.2f deg (eq. + %4.1f deg)"%(180-dynAdv,180-dynAdv-caVal))
print("    dyn. c.a. (rec.): %6.2f deg (eq. - %4.1f deg)"%(dynRec,caVal-dynRec))


#### Plotting section
# Do the plotting, save figure
plt.figure(1, figsize=(5.8, 6.75), dpi=80)
plt.plot(Xfit,Yfit,'m:',label="Polynomial fit")
for i in range(nMD):
    plt.plot(MDdataAvg[i][:,0],MDdataAvg[i][:,1],CurveSpec[i],label=MDlegend[i])
plt.plot([-12,12],[yWb,yWb],'r--',label="Wall locations")
plt.plot([-12,12],[yWt,yWt],'r--')
plt.title("Interface shape")
plt.xlabel("x [nm]")
plt.ylabel("y [nm]")
# plt.legend(loc="center left")
plt.axis("equal")
# plt.ylim([0.0,30.0])
# plt.xlim([-2.0,5.0])
plt.savefig("forPaper_steadyInterface_wMcent_eqAxis_shearEq.pdf", format="pdf")



# Plot the interface angle data
# plt.figure(2, figsize=(8.0, 4.75), dpi=80)
plt.figure(2, figsize=(9.5, 6.75), dpi=80)
ax = plt.axes()
for i in range(nMD):
    plt.plot(MDangleArr[i][:,0],MDangleArr[i][:,2]/np.pi*180.0,CurveSpec[i],label=MDlegend[i])
plt.plot(AfitY,Afit/np.pi*180.0,'m:',label="Polynomial fit")
plt.plot(AfitY,Afit1/np.pi*180.0,'m:')
plt.plot(AfitY,Afit2/np.pi*180.0,'m:')
# plt.plot(AfitY,Afit3/np.pi*180.0,'m:')
plt.scatter(np.array([yWb,yWt]),np.array([dynRec,dynAdv]),c='m',marker='x')
plt.scatter(np.array([yWb,yWt]),np.array([dynRec1,dynAdv1]),c='m',marker='x')
plt.scatter(np.array([yWb,yWt]),np.array([dynRec2,dynAdv2]),c='m',marker='x')
plt.scatter(np.array([yWbR,yWtR]),np.array([dynRecR,dynAdvR]),c='g',marker='x')
plt.plot([0 ,10],[    caVal      ,    caVal      ],'k:')
plt.plot([20,30],[180-caVal      ,180-caVal      ],'k:')
plt.plot([yWb,yWb],[caVal-18,caVal+2],'r--',label="Wall locations")
plt.plot([yWt,yWt],[180-caVal-18,180-caVal+2],'r--')
plt.title("Interface angle, MD, %s, %s"%(CaNum,ChNum))
ax.text(0.36, 0.852, r"$\theta_0$", transform=ax.transAxes)
ax.text(0.74, 0.553, r"$180-\theta_0$", transform=ax.transAxes)
plt.xlabel("y [nm]")
plt.ylabel(r"$\theta$ [deg]")
plt.legend(loc="center")
# plt.axis("equal")
plt.ylim([58.0,100.0])
plt.savefig("forPaper_steadyInterface_wMcent_angDist_y_ShearEquil_v2.pdf", format="pdf")


plt.show()
