#!/usr/bin/python3

import numpy as np
import matplotlib.pyplot as plt
import sys

plt.rcParams['text.usetex']=True
plt.rcParams.update({'figure.autolayout': True})  # to make sure that labels are inside printing area

font = {'family' : 'DejaVu Sans',
        'weight' : 'bold',
        'size'   : 18}

plt.rc('font', **font)

# Bounding coordinates for averaging liquid water density [nm]
xbL = np.array([65,93])

# Bounding coordinates for averaging vapour water density [nm]
xbV = np.array([55,104])

# Read the bin array data
Xar = np.loadtxt("../02_Xmatrix.txt")
Yar = np.loadtxt("../02_Ymatrix.txt")
Nx  = np.shape(Xar)[0]
Ny  = np.shape(Xar)[1]

print("Read file with Nx = %d, Ny = %d bins! Measurement units: nm"%(Nx,Ny))

# Read the density array
Mar = np.loadtxt("../02_average_wMcent_Mmatrix.txt")

# Display the minimum and maximum binning coordinates
bin_dx = Xar[1,0]-Xar[0,0]
bin_dy = Yar[0,1]-Yar[0,0]
print("x in [%f, %f], dx = %f"%(Xar[0,0],Xar[Nx-1,0],bin_dx))
print("y in [%f, %f], dy = %f"%(Yar[0,0],Yar[0,Ny-1],bin_dy))

# # Bin volume in m^3
# binVol = bin_dx*bin_dy*4.67650*1e-27
# # Atomic mass unit normalisation of M
# binVol = binVol/1.6605402e-27

# Bin volume in m^3 x1e27
binVol = bin_dx*bin_dy*4.67650
# Atomic mass unit normalisation of M
binVol = binVol/1.6605402

# Do the average in the liquid part
indL = (Xar[:,0]<xbL[0]).nonzero()[0][-1]
indR = (Xar[:,0]>xbL[1]).nonzero()[0][ 0]
print(" Found liquid indices %d and %d!"%(indL,indR))
avgRhoL = np.sum(Mar[indL:indR,:],axis=0)/len(Mar[indL:indR,0])

# Do the average in the vapour part
indL = (Xar[:,0]<xbV[0]).nonzero()[0][-1]
indR = (Xar[:,0]>xbV[1]).nonzero()[0][ 0]
print(" Found vapour indices %d and %d!"%(indL,indR))
avgRhoV = (np.sum(Mar[:indL,:],axis=0)+np.sum(Mar[indR:,:],axis=0))/(len(Mar[:indL,0])+len(Mar[indR:,0]))

# Do the whole domain averages and print result to screen
DavgRhoL = np.sum(avgRhoL[4:-4])/len(avgRhoL[4:-4])
DavgRhoV = np.sum(avgRhoV[3:-3])/len(avgRhoV[3:-3])
print(" Obtained liquid density average: %.2f"%DavgRhoL)
print(" Obtained vapour density average: %.6f"%DavgRhoV)

# Do the plotting, save figure
plt.figure(1, figsize=(4.0, 6.75), dpi=160)
plt.plot(avgRhoL,Yar[0,:],'k-x',label="MD, bottom wall")
plt.title("Liquid density distribution")
plt.xlabel(r"$\rho_l$ [kg/m$^3$]")
plt.ylabel("y [nm]")
# plt.legend(loc="lower right")
plt.ylim([-0.08,1.85])
# plt.xlim([0.0,32000])
plt.savefig("plotLiqDens_vs_y.pdf", format="pdf")

# Save in data file
SvData = np.zeros((len(Yar[0,:]),2))
SvData[:,0] = Yar[0,:]
SvData[:,1] = avgRhoL
np.savetxt("plotLiqDens_vs_y.txt",SvData,header="y [nm], rho_L [kg/m3]")

plt.figure(2, figsize=(4, 6.75), dpi=160)
plt.plot(avgRhoV,Yar[0,:],'g-',label="MD, bottom wall")
plt.title("Vapour density (averaged)")
plt.xlabel(r"$\rho_{v}$ [kg/m$^3$]")
plt.ylabel("y [nm]")
# plt.legend(loc="lower right")
# plt.ylim([-1.5,5.5])
# plt.xlim([0.0,32000])
# plt.savefig("plotUvel_yind%03d.pdf"%Ny, format="pdf")

plt.figure(3, figsize=(4, 6.75), dpi=160)
plt.plot(avgRhoV,Yar[0,:],'g-',label="MD, bottom wall")
plt.title("Vapour density (averaged)")
plt.xlabel(r"$\rho_{v}$ [kg/m$^3$]")
plt.ylabel("y [nm]")
# plt.legend(loc="lower right")
# plt.ylim([-1.5,5.5])
plt.xlim([-0.001,0.01])
# plt.savefig("plotUvel_yind%03d.pdf"%Ny, format="pdf")


plt.show()
